module Pages.Services.Elm exposing (Model, Msg, page)

import Config.Helpers.Cards.Helpers exposing (cardMaker)
import Config.Helpers.Cards.Types as C
import Config.Helpers.Format exposing (..)
import Config.Helpers.Headers.Helpers exposing (..)
import Config.Helpers.Headers.Types exposing (Header)
import Config.Helpers.Response
    exposing
        ( pageList
        , topLevelContainer
        )
import Config.Helpers.ServiceFormat exposing (..)
import Config.Helpers.ToolTip exposing (..)
import Config.Helpers.Viewport exposing (resetViewport)
import Config.Pages.Services.Records.ElmBuilds exposing (servicesElmBuilds)
import Config.Style.Colour exposing (..)
import Config.Style.Transitions
    exposing
        ( hoverFontDarkOrange
        , transitionStyleFast
        , transitionStyleSlow
        )
import Effect exposing (Effect)
import Element as E exposing (..)
import Element.Background as B
import Element.Border as D
import Element.Font as F
import Html
import Html.Attributes as H exposing (style)
import Layouts
import Page exposing (Page)
import Route exposing (Route)
import Route.Path as Path
import Shared
import View exposing (View)


page : Shared.Model -> Route () -> Page Model Msg
page shared route =
    Page.new
        { init = init
        , update = update
        , subscriptions = subscriptions
        , view = view shared
        }
        |> Page.withLayout toLayout


toLayout : Model -> Layouts.Layout Msg
toLayout model =
    Layouts.Navbar {}



-- INIT


type alias Model =
    {}


init : () -> ( Model, Effect Msg )
init () =
    ( {}
    , Effect.map
        (\_ -> NoOp)
        (Effect.sendCmd resetViewport)
    )



-- UPDATE


type Msg
    = NoOp


update : Msg -> Model -> ( Model, Effect Msg )
update msg model =
    case msg of
        NoOp ->
            ( model
            , Effect.none
            )



-- SUBSCRIPTIONS


subscriptions : Model -> Sub Msg
subscriptions model =
    Sub.none



-- VIEW


view : Shared.Model -> Model -> View Msg
view shared model =
    { title = "services (elmBuilds)"
    , attributes = []
    , element = elmBuildsContainer shared.device
    }


elmBuildsContainer : Device -> Element msg
elmBuildsContainer device =
    topLevelContainer (elmBuildsList device)


elmBuildsList : Device -> Element msg
elmBuildsList device =
    column
        (case ( device.class, device.orientation ) of
            _ ->
                pageList device
        )
    <|
        List.concat
            [ List.map
                (\service ->
                    cardMaker device (C.ServicePage service) contentList
                )
                [ servicesElmBuilds ]
            ]


contentList : List (Element msg)
contentList =
    [ column
        bodyFormat
        [ chunkMaker servicesElmBuilds.serviceArticle.articleParagraph.paragraph1
        , chunkMaker servicesElmBuilds.serviceArticle.articleParagraph.paragraph2
        , chunkMaker servicesElmBuilds.serviceArticle.articleParagraph.paragraph3
        , titleMaker servicesElmBuilds.serviceArticle.articleTitles.title1
        , numberMaker servicesElmBuilds.serviceArticle.articleListEntries.list1
        , titleMaker servicesElmBuilds.serviceArticle.articleTitles.title2
        , bulletPointMaker servicesElmBuilds.serviceArticle.articleListEntries.list2
        , titleMaker servicesElmBuilds.serviceArticle.articleTitles.title3
        , elmCodeRenderer
        ]
    ]


elmCodeRenderer : Element msg
elmCodeRenderer =
    el
        [ paddingEach
            { top = 15
            , bottom = 15
            , left = 20
            , right = 20
            }
        , B.color colourTheme.backgroundLightGrey
        , D.rounded 10
        , width fill
        , spacing 8
        ]
    <|
        column
            [ F.size 14
            , spacing 5
            , width fill
            ]
            [ renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 0 0 ]
                , text "cardImageMaker"
                    |> el [ F.color syntaxTheme.key ]
                , text " : "
                    |> el [ F.color syntaxTheme.operator ]
                , text "String"
                    |> el [ F.color syntaxTheme.keyword ]
                , text " -> "
                    |> el [ F.color syntaxTheme.operator ]
                , text "String"
                    |> el [ F.color syntaxTheme.keyword ]
                , text " -> "
                    |> el [ F.color syntaxTheme.operator ]
                , text "Element"
                    |> el [ F.color syntaxTheme.keyword ]
                , text " msg"
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 0 0 ]
                , text "cardImageMaker"
                    |> el [ F.color syntaxTheme.key ]
                , text " image blurb"
                , text " ="
                    |> el [ F.color syntaxTheme.operator ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 16 0 ]
                , text "let"
                    |> el [ F.color syntaxTheme.keyword ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 32 0 ]
                , text "mobileImageSize"
                , text " ="
                    |> el [ F.color syntaxTheme.operator ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 48 0 ]
                , text "["
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " E"
                , text "."
                    |> el [ F.color syntaxTheme.punctuation ]
                , text "width"
                , text " <| "
                    |> el [ F.color syntaxTheme.operator ]
                , text "px"
                , text " 45"
                    |> el [ F.color syntaxTheme.string ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 48 0 ]
                , text ","
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " E"
                , text "."
                    |> el [ F.color syntaxTheme.punctuation ]
                , text "height"
                , text " <| "
                    |> el [ F.color syntaxTheme.operator ]
                , text "px"
                , text " 45"
                    |> el [ F.color syntaxTheme.string ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 48 0 ]
                , text "]"
                    |> el [ F.color syntaxTheme.punctuation ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 16 0 ]
                , text "in"
                    |> el [ F.color syntaxTheme.keyword ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 16 0 ]
                , text "el"
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 32 0 ]
                , text "["
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " alignRight"
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 32 0 ]
                , text ","
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " alignTop"
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 32 0 ]
                , text ","
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " padding"
                , text " 10"
                    |> el [ F.color syntaxTheme.string ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 32 0 ]
                , text "]"
                    |> el [ F.color syntaxTheme.punctuation ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 16 0 ]
                , text "<|"
                    |> el [ F.color syntaxTheme.operator ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 32 0 ]
                , text "el"
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 48 0 ]
                , text "["
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " D"
                , text "."
                    |> el [ F.color syntaxTheme.punctuation ]
                , text "rounded"
                , text " 100"
                    |> el [ F.color syntaxTheme.string ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 48 0 ]
                , text ","
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " D"
                , text "."
                    |> el [ F.color syntaxTheme.punctuation ]
                , text "width"
                , text " 5"
                    |> el [ F.color syntaxTheme.string ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 48 0 ]
                , text ","
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " glowDeepDarkGrey"
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 48 0 ]
                , text ","
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " D"
                , text "."
                    |> el [ F.color syntaxTheme.punctuation ]
                , text "color"
                , text " colourTheme"
                , text "."
                    |> el [ F.color syntaxTheme.punctuation ]
                , text "backgroundDarkGrey"
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 48 0 ]
                , text ","
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " B"
                , text "."
                    |> el [ F.color syntaxTheme.punctuation ]
                , text "color"
                , text " colourTheme"
                , text "."
                    |> el [ F.color syntaxTheme.punctuation ]
                , text "backgroundDarkGrey"
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 48 0 ]
                , text "]"
                    |> el [ F.color syntaxTheme.punctuation ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 32 0 ]
                , text "<|"
                    |> el [ F.color syntaxTheme.operator ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 48 0 ]
                , text "E"
                , text "."
                    |> el [ F.color syntaxTheme.punctuation ]
                , text "image"
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 64 0 ]
                , text "("
                    |> el [ F.color syntaxTheme.punctuation ]
                , text "["
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " alignRight"
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 64 0 ]
                , text ","
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " alignTop"
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 64 0 ]
                , text ","
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " D"
                , text "."
                    |> el [ F.color syntaxTheme.punctuation ]
                , text "rounded"
                , text " 100"
                    |> el [ F.color syntaxTheme.string ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 64 0 ]
                , text ","
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " clip"
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 64 0 ]
                , text "]"
                    |> el [ F.color syntaxTheme.punctuation ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 64 0 ]
                , text "++"
                    |> el [ F.color syntaxTheme.operator ]
                , text " ("
                    |> el [ F.color syntaxTheme.punctuation ]
                , text "case"
                    |> el [ F.color syntaxTheme.keyword ]
                , text " ("
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " device"
                , text "."
                    |> el [ F.color syntaxTheme.punctuation ]
                , text "class"
                , text ","
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " device"
                , text "."
                    |> el [ F.color syntaxTheme.punctuation ]
                , text "orientation"
                , text " )"
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " of"
                    |> el [ F.color syntaxTheme.keyword ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 80 0 ]
                , text "("
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " Phone"
                , text ","
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " Portrait"
                , text " )"
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " ->"
                    |> el [ F.color syntaxTheme.operator ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 96 0 ]
                , text "mobileImageSize"
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 80 0 ]
                , text "("
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " Tablet"
                , text ","
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " Portrait"
                , text " )"
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " ->"
                    |> el [ F.color syntaxTheme.operator ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 96 0 ]
                , text "mobileImageSize"
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 80 0 ]
                , text "_"
                , text " ->"
                    |> el [ F.color syntaxTheme.operator ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 96 0 ]
                , text "["
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " E"
                , text "."
                    |> el [ F.color syntaxTheme.punctuation ]
                , text "width"
                , text " <| "
                    |> el [ F.color syntaxTheme.operator ]
                , text "px"
                , text " 90"
                    |> el [ F.color syntaxTheme.string ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 96 0 ]
                , text ","
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " E"
                , text "."
                    |> el [ F.color syntaxTheme.punctuation ]
                , text "height"
                , text " <| "
                    |> el [ F.color syntaxTheme.operator ]
                , text "px"
                , text " 90"
                    |> el [ F.color syntaxTheme.string ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 96 0 ]
                , text "]"
                    |> el [ F.color syntaxTheme.punctuation ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 64 0 ]
                , text ")"
                    |> el [ F.color syntaxTheme.punctuation ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 64 0 ]
                , text ")"
                    |> el [ F.color syntaxTheme.punctuation ]
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 64 0 ]
                , text "{"
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " src"
                , text " = "
                    |> el [ F.color syntaxTheme.operator ]
                , text "image"
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 64 0 ]
                , text ","
                    |> el [ F.color syntaxTheme.punctuation ]
                , text " description"
                , text " = "
                    |> el [ F.color syntaxTheme.operator ]
                , text "blurb"
                ]
            , renderCodeLine syntaxTheme
                [ text ""
                    |> el [ paddingXY 64 0 ]
                , text "}"
                    |> el [ F.color syntaxTheme.punctuation ]
                ]
            ]
